<?php
/**
 * CAPTCHA image
 * @copyright 2017 ishitaka
 * @author ishitaka
 */

session_start();

// 画像の幅
$captcha_image_width = 100;
// 画像の高さ
$captcha_image_height = 36;
// 文字数
$captcha_string_length = 4;

// Prefix
$prefix = isset( $_GET['prefix'] ) && preg_match( '/^[a-zA-Z0-9]{1,20}$/ ',$_GET['prefix'] ) ? $_GET['prefix'] : 'default';

// 文字モード（'en' または 'jp'）
$char_mode = isset( $_GET['char_mode'] ) ? $_GET['char_mode'] : 'en';

// 文字化け防止に、文字コードを明示的に指定
mb_internal_encoding( 'UTF-8' );

// ダミー文字用 TrueType フォント テーブル
$dummy_fonts = array(
	dirname( __FILE__ ) . '/fonts/mplus/mplus-1c-thin-captcha.ttf',
	dirname( __FILE__ ) . '/fonts/mplus/mplus-1c-light-captcha.ttf',
);

// 文字用 TrueType フォント テーブル
$string_fonts = array(
	dirname( __FILE__ ) . '/fonts/mplus/mplus-1c-regular-captcha.ttf',
	dirname( __FILE__ ) . '/fonts/mplus/mplus-1c-medium-captcha.ttf',
	dirname( __FILE__ ) . '/fonts/mplus/mplus-1c-bold-captcha.ttf',
	dirname( __FILE__ ) . '/fonts/mplus/mplus-1c-heavy-captcha.ttf',
);

// ダミー文字列テーブル
$dummy_string_table = '＃＊○◎△▽☆';

if ( $char_mode === 'jp' ) {
	// 文字列テーブル
	$string_table = 'あいうえおかきくけこさしすせそたちつてとなにぬねのはひふほまみむめもらりるれろわ';
	// 基準のフォント サイズ
	$captcha_font_size = 18;
} else {
	// 文字列テーブル
	$string_table = 'abcdefghjkmnpstuvwyz23456789';
	// 基準のフォント サイズ
	$captcha_font_size = 21;
}

// 乱数のシードを設定
//mt_srand( hexdec( bin2hex( openssl_random_pseudo_bytes( 4 ) ) ) );
mt_srand();

// ダミー文字列を生成
$dummy_string = '';
for ( $i = 0; $i < $captcha_string_length + 1; $i++ ) {
	$dummy_string .= mb_substr( $dummy_string_table, mt_rand( 0, mb_strlen( $dummy_string_table ) - 1 ), 1 );
}

// 文字列を生成
$string = '';
for( $i = 0; $i < $captcha_string_length; $i++ ) {
	$string .= mb_substr( $string_table, mt_rand( 0, mb_strlen( $string_table ) - 1 ), 1 );
}

// 画像表示した文字列(正解)をセッション変数に入れる
$_SESSION["xo_security_captcha_$prefix"] = $string;

// 画像サイズを指定して、画像オブジェクトを生成
$img = imagecreate( $captcha_image_width, $captcha_image_height );

// 背景色を指定(白色)
imagecolorallocate( $img, 255, 255, 255 );

$gd_info = gd_info();

// ダミー文字を描画
for ( $i = 0; $i < mb_strlen( $dummy_string ); $i++ ) {
	$x = ( $captcha_image_width / mb_strlen( $dummy_string ) ) * $i + mt_rand( -$captcha_font_size / 15, $captcha_font_size / 15 );
	$y = ( $captcha_image_height + $captcha_font_size) / 2 + mt_rand( -$captcha_image_height / 5, $captcha_image_height / 5 );
	$c = mt_rand( 32, 128 );

	if ( $gd_info['JIS-mapped Japanese Font Support'] ) {
		$char = mb_convert_encoding( mb_substr( $dummy_string, $i, 1 ), 'SJIS', 'UTF-8' );
	} else {
		$char = mb_substr( $dummy_string, $i, 1 );
	}

	imagettftext(
		$img,
		mt_rand( $captcha_font_size * 1.2, $captcha_font_size * 1.4 ),
		mt_rand( -45, 45 ),
		$x,
		$y,
		imagecolorallocate( $img, $c, $c, $c ),
		$dummy_fonts[mt_rand( 0, count( $dummy_fonts ) - 1 )],
		$char
	);
}

// CPATCHA 文字を描画
for ( $i = 0; $i < mb_strlen( $string ); $i++ ) {
	$x = ( ( $captcha_image_width - 8 ) / mb_strlen( $string ) ) * $i + mt_rand( -$captcha_font_size / 10, $captcha_font_size / 10 ) + 4;
	$y = ( $captcha_image_height + $captcha_font_size) / 2 + mt_rand( -$captcha_image_height / 10 , $captcha_image_height / 10 );
	$c = mt_rand( 0, 64 );

	if ( $gd_info['JIS-mapped Japanese Font Support'] ) {
		$char = mb_convert_encoding( mb_substr( $string, $i, 1 ), 'SJIS', 'UTF-8' );
	} else {
		$char = mb_substr( $string, $i, 1 );
	}

	imagettftext(
		$img,
		mt_rand( $captcha_font_size * 0.8, $captcha_font_size * 1.2 ),
		mt_rand( -20, 20 ),
		$x,
		$y,
		imagecolorallocate( $img, $c, $c, $c ),
		$string_fonts[mt_rand( 0, count( $string_fonts ) - 1 )],
		$char
	);
}

// 画像を出力
header( 'Cache-control: no-cache' );
header( 'Content-type: image/png' );
imagepng( $img );

// 画像の破棄（リソースの解放）
imagedestroy( $img );
