<?php

require_once( __DIR__ . '/class-log-list-table.php' );

/**
 * XO Security admin class.
 *
 * @since 1.0.0
 */
class XO_Security_Admin {
	private $parent;
	private $list_table;
	private $settings_page;
	private $loginfile_content = "<?php require_once './wp-login.php'; ?>";

	/**
	 * Construction.
	 * 
	 * @since 1.0.0
	 * 
	 * @param XO_Security $parent XO_Security object.
	 */
	function __construct( $parent ) {
		$this->parent = $parent;
		add_action( 'plugins_loaded', array( $this, 'setup' ), 99999 );
	}

	function setup() {
		add_action( 'admin_bar_init', array( $this, 'admin_bar_init' ), 9999 );
		add_action( 'admin_menu', array( $this, 'add_admin_menu' ) );
		add_filter( 'set-screen-option', array( $this, 'set_screen_option' ), 10, 3 );
		add_action( 'admin_init', array( $this, 'option_page_init' ) );
		add_action( 'wp_dashboard_setup', array( $this, 'dashboard_setup' ) );
		add_filter( 'plugin_action_links', array( $this, 'plugin_action_links' ), 10, 2 );
	}

	private function get_option() {
		$options = get_option( 'xo_security_options' );

		if ( ! is_main_site() ) {
			switch_to_blog( 1 );
			$mainsite_options = get_option( 'xo_security_options' );
			restore_current_blog();

			$options['ms_common_login_page'] = isset( $mainsite_options['ms_common_login_page'] ) ? $mainsite_options['ms_common_login_page'] : false;
			if ( $options['ms_common_login_page'] ) {
				$options['login_page'] = isset( $mainsite_options['login_page'] ) ? $mainsite_options['login_page'] : false;
				$options['login_page_name'] = isset( $mainsite_options['login_page_name'] ) ? $mainsite_options['login_page_name'] : '';
			}
		}

		return $options;
	}

	function author_rewrite_rules() {
		return array();
	}

	function admin_bar_init() {
		wp_enqueue_style( 'xo-security-admin', XO_SECURITY_URL . '/css/admin.css', false, XO_SECURITY_VERSION );
	}

    function plugin_action_links( $actions, $plugin_file ) {
		if ( 'xo-security.php' == basename( $plugin_file ) ) {
			$settings = array(
				'settings' => '<a href="' . admin_url( 'options-general.php?page=xo-security-settings' ) . '">' . __( 'Settings', 'xo-security' ) . '</a>',
			);
			$actions = array_merge( $settings, $actions );
		}
		return $actions;
	}

	/**
	 * Admin screen Add menu.
	 */
	function add_admin_menu() {
		$login_log_page = add_submenu_page( 'profile.php', __( 'Login log', 'xo-security' ), __( 'Login log', 'xo-security' ), 'level_2', 'xo-security-login-log', array( $this, 'login_log_page' ) );
		add_action( "load-{$login_log_page}", array( $this, 'add_login_log_page_tabs' ) );

		$this->settings_page = add_options_page( 'XO Security', 'XO Security', 'manage_options', 'xo-security-settings', array( $this, 'option_page' ) );
		add_action( "load-{$this->settings_page}", array( $this, 'add_settings_page_tabs' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
	}

	/**
	 * Add a script.
	 */
	function admin_enqueue_scripts( $hook ) {
		if ( $this->settings_page === $hook ) {
			wp_enqueue_script( 'xo-security-admin', XO_SECURITY_URL . '/js/admin.js', array( 'jquery' ), XO_SECURITY_VERSION );	
			wp_add_inline_script( 'xo-security-admin', 'var xo_security_site_url = "' . site_url( '/' ) . '";', 'before' );
		}
	}

	/**
	 * Add a tab to the login log page.
	 */
	function add_login_log_page_tabs() {
		add_screen_option( 'per_page', array( 'label' => __( 'Number of items per page:', 'xo-security' ), 'default' => 100, 'option' => 'login_log_per_page' ) );

		$screen = get_current_screen();
		$screen->add_help_tab( array(
			'id' => 'loginlogs-help',
			'title' => __( 'Overview', 'xo-security' ),
			'content' =>
				'<p>' . __( 'View the log of login.', 'xo-security' ). '</p>'. 
				'<p>' . __( 'If the user name is the same as an existing user, the password is not displayed (not recorded in the log itself).', 'xo-security' ) . '</p>',
		) );

		$this->list_table = new XO_Login_Log_List_Table();
		$this->list_table->default_status = isset( $this->parent->options['log_default_status'] ) ? $this->parent->options['log_default_status'] : '';
	}

	/**
	 * Add a tab to the setting page.
	 */
	function add_settings_page_tabs() {
		$screen = get_current_screen();
		$screen->add_help_tab( array(
			'id' => 'overview',
			'title' => __( 'Overview', 'xo-security' ),
			'content' =>
				'<p>' . __( 'XO Security setup.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Limit of records and login attempts of login log, change the login page, disable such as XML-RPC, it makes the security-related settings.', 'xo-security' ) . '</p>'
		) );
		$screen->add_help_tab( array(
			'id' => 'login-config',
			'title' => __( 'Login', 'xo-security' ),
			'content' =>
				'<p>' . __( 'Make login related settings.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Number of trials restriction limits the number of times that you can try from the same IP address during a specified time. From was over a specified number of IP address you will not be able to log in.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Login page, Change the name of the login page.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Login alert, Send an e-mail when login.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Login CAPTCHA, add CAPTCHA to the login page.', 'xo-security' ) . '</p>'
		) );
		$screen->add_help_tab( array(
			'id' => 'comment-config',
			'title' => __( 'Comment', 'xo-security' ),
			'content' =>
				'<p>' . __( 'Make comment related settings.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Comment CAPTCHA, add CAPTCHA to the comment form.', 'xo-security' ) . '</p>'
		) );
		$screen->add_help_tab( array(
			'id' => 'xml-rpc-config',
			'title' => __( 'XML-RPC', 'xo-security' ),
			'content' =>
				'<p>' . __( 'Disable XML-RPC, XML-RPC functionality disabled. This setting does not disable XML-RPC pingback.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Disabling pingback XML-RPC, XML-RPC pingback functionality disabled.', 'xo-security' ) . '</p>'
		) );
		$screen->add_help_tab( array(
			'id' => 'rest-api-config',
			'title' => __( 'REST API', 'xo-security' ),
			'content' =>
				'<p>' . __( 'Disable REST API, REST API functionality disabled.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'REST API URL prefix, Change the prefix for the REST API URL.', 'xo-security' ) . '</p>'
		) );
		$screen->add_help_tab( array(
			'id' => 'secret-config',
			'title' => __( 'Secret', 'xo-security' ),
			'content' =>
				'<p>' . __( 'Disabling author archives the author archive page does not appear.', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Deleting the comment author name class deletes the class that contains the username that is added to the comment list tag.', 'xo-security' ) . '</p>'
		) );
		$screen->add_help_tab( array(
			'id' => 'environment-config',
			'title' => __( 'Environment', 'xo-security' ),
			'content' =>
				'<p>' . __( 'Select the method of acquiring the IP address. Normally select "Auto".', 'xo-security' ) . '</p>' .
				'<p>' . __( 'Automatic deletion of login log automatically deletes the old log in the log of the previous period specified.', 'xo-security' ) . '</p>'
		) );
	}

	/**
	 * Processing when setting options on the login log page.
	 */
	function set_screen_option( $status, $option, $value ) {
		if ( 'login_log_per_page' == $option ) {
			$new_value = (int) $value;
			if ( $new_value >= 1 && $new_value <= 999 ) {
				return $new_value;
			}
			return $status;
		}
	}

	/**
	 * Display the login log page.
	 */
	function login_log_page() {
		$this->list_table->prepare_items();
	?>
		<div class="wrap">
			<div id="icon-profile" class="icon32"></div>
			<h1><?php _e( 'Login log', 'xo-security' ); ?></h1>
			<form id="loginlogs-filter" method="get" action="">
				<input type="hidden" name="page" value="<?php echo esc_attr( $_REQUEST['page'] ); ?>" />
				<?php $this->list_table->display(); ?>
			</form>
		</div>
	<?php
	}

	function option_page() {
		$this->parent->options = $this->get_option();

		$active_tab = isset( $_GET['tab'] ) ? $_GET['tab'] : 'status-page';

		echo '<div class="wrap">';
		echo '<h1>' . __( 'XO Security Settings', 'xo-security' ) . '</h1>';
		echo '<h2 class="nav-tab-wrapper">';
		echo '<a href="?page=xo-security-settings&amp;tab=status-page" class="nav-tab ' . ( $active_tab == 'status-page' ? 'nav-tab-active' : '' ) . '">' . __( 'Status', 'xo-security' ) . '</a>';
		echo '<a href="?page=xo-security-settings&amp;tab=login-page" class="nav-tab ' . ( $active_tab == 'login-page' ? 'nav-tab-active' : '' ) . '">' . __( 'Login', 'xo-security' ) . '</a>';
		echo '<a href="?page=xo-security-settings&amp;tab=comment-page" class="nav-tab ' . ( $active_tab == 'comment-page' ? 'nav-tab-active' : '' ) . '">' . __( 'Comment', 'xo-security' ) . '</a>';
		echo '<a href="?page=xo-security-settings&amp;tab=xmlrpc-page" class="nav-tab ' . ( $active_tab == 'xmlrpc-page' ? 'nav-tab-active' : '' ) . '">' . __( 'XML-RPC', 'xo-security' ) . '</a>';
		echo '<a href="?page=xo-security-settings&amp;tab=restapi-page" class="nav-tab ' . ( $active_tab == 'restapi-page' ? 'nav-tab-active' : '' ) . '">' . __( 'REST API', 'xo-security' ) . '</a>';
		echo '<a href="?page=xo-security-settings&amp;tab=secret-page" class="nav-tab ' . ( $active_tab == 'secret-page' ? 'nav-tab-active' : '' ) . '">' . __( 'Secret', 'xo-security' ) . '</a>';
		echo '<a href="?page=xo-security-settings&amp;tab=environment-page" class="nav-tab ' . ( $active_tab == 'environment-page' ? 'nav-tab-active' : '' ) . '">' . __( 'Environment', 'xo-security' ) . '</a>';
		echo '</h2>';
		echo '<form method="post" action="options.php">';

		switch ( $active_tab ){
			case 'status-page':
				$this->status_page();
				break;
			case 'login-page':
				settings_fields( 'xo_security_login' );
				do_settings_sections( 'xo_security_login' );
				submit_button();
				break;
			case 'comment-page':
				settings_fields( 'xo_security_comment' );
				do_settings_sections( 'xo_security_comment' );
				submit_button();
				break;
			case 'xmlrpc-page':
				settings_fields( 'xo_security_xmlrpc' );
				do_settings_sections( 'xo_security_xmlrpc' );
				submit_button();
				break;
			case 'restapi-page':
				settings_fields( 'xo_security_restapi' );
				do_settings_sections( 'xo_security_restapi' );
				submit_button();
				break;
			case 'secret-page':
				settings_fields( 'xo_security_secret' );
				do_settings_sections( 'xo_security_secret' );
				submit_button();
				break;
			case 'environment-page':
				settings_fields( 'xo_security_environment' );
				do_settings_sections( 'xo_security_environment' );
				submit_button();
				break;
		}

		echo '</form>';
		echo '</div>';
	}

	function status_page() {
		global $wp_version, $wpdb;

		$interval = isset( $this->parent->options['interval'] ) ? $this->parent->options['interval'] : '0';
		$login_page = isset( $this->parent->options['login_page'] ) ? $this->parent->options['login_page'] : false;
		$login_alert = isset( $this->parent->options['login_alert'] ) ? $this->parent->options['login_alert'] : false;
		$error_login_message = isset( $this->parent->options['error_login_message'] ) ? $this->parent->options['error_login_message'] : false;
		$login_id_type = isset( $this->parent->options['login_id_type'] ) ? $this->parent->options['login_id_type'] : '';
		$login_captcha = isset( $this->parent->options['login_captcha'] ) ? ( $this->parent->options['login_captcha'] != '' ) : false;
		$comment_captcha = isset( $this->parent->options['comment_captcha'] ) ? ( $this->parent->options['comment_captcha'] != '' ) : false;
		$comment_spam = isset( $this->parent->options['comment_spam'] ) ? $this->parent->options['comment_spam'] : false;
		$comment_bot = isset( $this->parent->options['comment_bot'] ) ? $this->parent->options['comment_bot'] : false;
		$xmlrpc = isset( $this->parent->options['xmlrpc'] ) ? $this->parent->options['xmlrpc'] : false;
		$pingback = isset( $this->parent->options['pingback'] ) ? $this->parent->options['pingback'] : false;
		$rest = isset( $this->parent->options['rest'] ) ? $this->parent->options['rest'] : false;
		$edit_author_slug = isset( $this->parent->options['edit_author_slug'] ) ? $this->parent->options['edit_author_slug'] : false;
		$author_archive = isset( $this->parent->options['author_archive'] ) ? $this->parent->options['author_archive'] : false;
		$comment_author_class = isset( $this->parent->options['comment_author_class'] ) ? $this->parent->options['comment_author_class'] : false;
		$oembed_author = isset( $this->parent->options['oembed_author'] ) ? $this->parent->options['oembed_author'] : false;
		$disable_feed = isset( $this->parent->options['disable_feed'] ) ? $this->parent->options['disable_feed'] : false;
		$remove_version = isset( $this->parent->options['remove_version'] ) ? $this->parent->options['remove_version'] : false;
		?>
		<h3 class="label"><?php _e( 'Setting status', 'xo-security' ); ?></h3>
		<table class="xo-security-form-table">
			<tbody>
				<tr>
					<th scope="row" class="status-check"><span class="check-on"></span></th>
					<td class="status-title"><?php _e( 'Record login', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Record the login log.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $interval != '0' ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Infinite login attempts', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Block connections that repeat login failures.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $login_page ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Modify login page', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Change the name of the login page.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $login_id_type != '' ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Login ID type', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Limit your login ID to either username or email address.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( ( defined( 'XO_SECURITY_LANGUAGE_WHITE_LIST' ) && XO_SECURITY_LANGUAGE_WHITE_LIST !== '' ) ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Login language Restrictions', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Limit the languages that can be logged in.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $error_login_message ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Login error message', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Simplify login error messages.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $login_captcha ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Login CAPTCHA', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Add CAPTCHA to the login page.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $login_alert ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Login Alert', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Send a mail when you log in.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $comment_captcha ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Comment CAPTCHA', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Add CAPTCHA to the comment form.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $comment_spam ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Comment spam protection', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Protect comment from spam.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $comment_bot ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Comment bot protection', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Protect comment from bots.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $xmlrpc ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Disable XML-RPC', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Disable the XML-RPC.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $pingback ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Disable XML-RPC Pinback', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Disable XML-RPC pingback features.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $rest ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Disable REST API', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Disable the REST API.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $edit_author_slug ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Edit author slug', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'The author slug can be edited.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $author_archive ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Disable author archives', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Disable authors archive page.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $comment_author_class ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Delete comment author class', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Delete the class that contains the username that will be added to the comment list tag.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $oembed_author ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Remove oEmbed username', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Remove the username from the oEmbed response data.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $disable_feed ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Disable RSS and Atom feeds', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Disable RSS and Atom feeds.', 'xo-security' ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( $remove_version ? 'check-on' : 'check-off' ); ?>"></span></th>
					<td class="status-title"><?php _e( 'Delete version information', 'xo-security' ); ?></td>
					<td class="status-description"><?php _e( 'Remove the generator meta tags and WordPress versions such as links and script tags.', 'xo-security' ); ?></td>
				</tr>
			</tbody>
		</table>
		<?php
		$server_info = isset( $_SERVER['SERVER_SOFTWARE'] ) ? wp_unslash( $_SERVER['SERVER_SOFTWARE'] ) : '';
		$php_version = phpversion();
		$mysql_version = method_exists( $wpdb, 'db_version' ) ? $wpdb->db_version() : '';
		$timezone = date_default_timezone_get();
		$wp_debug = ( defined( 'WP_DEBUG' ) && WP_DEBUG );

		$extensions = get_loaded_extensions();
		$php_module = array();
		$php_module_description = 'mbstring, gd, session ...';
		if ( ! in_array( 'mbstring', $extensions) ) $php_module[] = 'mbstring';
		if ( ! in_array( 'gd', $extensions) ) $php_module[] = 'gd';
		if ( ! in_array( 'session', $extensions) ) $php_module[] = 'session';
		if ( count( $php_module ) > 0 ) {
			$php_module_description = sprintf( _n( 'The %s module is unavailable.', 'The %s modules is unavailable.', count( $php_module ), 'xo-security' ), implode( ', ', $php_module ) );
		}
		?>
		<h3 class="label"><?php _e( 'Site Information', 'xo-security' ); ?></h3>
		<table class="xo-security-form-table">
			<tbody>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo $server_info !== '' ? 'check-on' : 'check-off'; ?>"></span></th>
					<td class="status-title"><?php _e( 'Server information', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo esc_html( $server_info ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo version_compare( $wp_version, '4.7.0' ) >= 0 ? 'check-on' : 'check-ng'; ?>"></span></th>
					<td class="status-title"><?php _e( 'WordPress Version', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo esc_html( $wp_version ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo version_compare( $php_version, '5.2.4' ) >= 0 ? 'check-on' : 'check-ng'; ?>"></span></th>
					<td class="status-title"><?php _e( 'PHP Version', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo esc_html( $php_version ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo version_compare( $php_version, '5.0.0' ) >= 0 ? 'check-on' : 'check-ng'; ?>"></span></th>
					<td class="status-title"><?php _e( 'MySQL Version', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo esc_html( $mysql_version ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo ( count( $php_module ) > 0 ) ? 'check-ng' : 'check-on'; ?>"></span></th>
					<td class="status-title"><?php _e( 'PHP extension', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo esc_html( $php_module_description ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo $timezone === 'UTC' ? 'check-on' : 'check-ng'; ?>"></span></th>
					<td class="status-title"><?php _e( 'Default timezone', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo esc_html( $timezone ); ?></td>
				</tr>
				<tr>
					<th scope="row" class="status-check"><span class="<?php echo $wp_debug ? 'check-ng' : 'check-on'; ?>"></span></th>
					<td class="status-title"><?php _e( 'WordPress debugging mode', 'xo-security' ); ?></td>
					<td class="status-description"><?php echo $wp_debug ? __( 'Enabled', 'xo-security' ) : __( 'Disabled', 'xo-security' ); ?></td>
				</tr>
			</tbody>
		</table>
		<?php
	}

	function option_page_init() {
		if ( delete_transient( 'xo_security_flush_rewrite_rules' ) ) {
			flush_rewrite_rules( false );
		}

		register_setting( 'xo_security_login', 'xo_security_options', array( $this, 'login_sanitize' ) );
		add_settings_section( 'xo_security_login_limit_section', __( 'Login', 'xo-security' ), '__return_empty_string', 'xo_security_login' );
		add_settings_field( 'limit_count', __( 'Infinite attempts', 'xo-security' ), array( $this, 'option_page_field_limit_count' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_field( 'blocked_tarpit', __( 'Block time delay', 'xo-security' ), array( $this, 'option_page_field_blocked_tarpit' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_field( 'failed_tarpit', __( 'Failure time delay', 'xo-security' ), array( $this, 'option_page_field_failed_tarpit' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_field( 'login_page', __( 'Modify login page', 'xo-security' ), array( $this, 'option_page_field_login_page' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_field( 'login_id_type', __( 'Login ID type', 'xo-security' ), array( $this, 'option_page_field_login_id_type' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_field( 'login_languages', __( 'Login language Restrictions', 'xo-security' ), array( $this, 'option_page_field_login_languages' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_field( 'error_login_message', __( 'Login error message', 'xo-security' ), array( $this, 'option_page_field_error_login_message' ), 'xo_security_login', 'xo_security_login_limit_section' );
		add_settings_section( 'xo_security_login_form_section', __( 'Login Form', 'xo-security' ), '__return_empty_string', 'xo_security_login' );
		add_settings_field( 'login_captcha', __( 'CAPTCHA', 'xo-security' ), array( $this, 'option_page_field_login_captcha' ), 'xo_security_login', 'xo_security_login_form_section' );
		add_settings_field( 'password_reset', __( 'Password reset link', 'xo-security' ), array( $this, 'option_page_field_password_reset' ), 'xo_security_login', 'xo_security_login_form_section' );
		add_settings_field( 'login_site_link', __( 'Go to site link', 'xo-security' ), array( $this, 'option_page_field_login_site_link' ), 'xo_security_login', 'xo_security_login_form_section' );
		add_settings_section( 'xo_security_login_alert_section', __( 'Login Alert', 'xo-security' ), '__return_empty_string', 'xo_security_login' );
		add_settings_field( 'login_alert', __( 'Login Alert', 'xo-security' ), array( $this, 'option_page_field_login_alert' ), 'xo_security_login', 'xo_security_login_alert_section' );

		register_setting( 'xo_security_comment', 'xo_security_options', array( $this, 'comment_sanitize' ) );
		add_settings_section( 'xo_security_comment_section', __( 'Comment', 'xo-security' ), '__return_false', 'xo_security_comment' );
		add_settings_field( 'comment_captcha', __( 'CAPTCHA', 'xo-security' ), array( $this, 'option_page_field_comment_captcha' ), 'xo_security_comment', 'xo_security_comment_section' );
		add_settings_field( 'comment_spam', __( 'Spam protection filter', 'xo-security' ), array( $this, 'option_page_field_comment_spam' ), 'xo_security_comment', 'xo_security_comment_section' );
		add_settings_field( 'comment_spam_action', __( 'Spam comment', 'xo-security' ), array( $this, 'option_page_field_comment_spam_action' ), 'xo_security_comment', 'xo_security_comment_section' );
		add_settings_field( 'comment_bot', __( 'Bot protection checkbox', 'xo-security' ), array( $this, 'option_page_field_comment_bot' ), 'xo_security_comment', 'xo_security_comment_section' );

		register_setting( 'xo_security_xmlrpc', 'xo_security_options', array( $this, 'xmlrpc_sanitize' ) );
		add_settings_section( 'xo_security_xmlrpc_section', __( 'XML-RPC', 'xo-security' ), '__return_false', 'xo_security_xmlrpc' );
		add_settings_field( 'xmlrpc', __( 'Disable XML-RPC', 'xo-security' ), array( $this, 'option_page_field_xmlrpc' ), 'xo_security_xmlrpc', 'xo_security_xmlrpc_section' );
		add_settings_field( 'pingback', __( 'Disable XML-RPC Pinback', 'xo-security' ), array( $this, 'option_page_field_pingback' ), 'xo_security_xmlrpc', 'xo_security_xmlrpc_section' );

		register_setting( 'xo_security_restapi', 'xo_security_options', array( $this, 'restapi_sanitize' ) );
		add_settings_section( 'xo_security_rest_section', __( 'REST API', 'xo-security' ), '__return_false', 'xo_security_restapi' );
		add_settings_field( 'rest', __( 'Disable REST API', 'xo-security' ), array( $this, 'option_page_field_rest' ), 'xo_security_restapi', 'xo_security_rest_section' );
		add_settings_field( 'rest_rename', __( 'Change REST API URL prefix', 'xo-security' ), array( $this, 'option_page_field_rest_rename' ), 'xo_security_restapi', 'xo_security_rest_section' );

		register_setting( 'xo_security_secret', 'xo_security_options', array( $this, 'secret_sanitize' ) );
		add_settings_section( 'xo_security_security_user_section', __( 'Username', 'xo-security' ), '__return_false', 'xo_security_secret' );
		add_settings_field( 'edit_author_slug', __( 'Edit author slug', 'xo-security' ), array( $this, 'option_page_field_edit_author_slug' ), 'xo_security_secret', 'xo_security_security_user_section' );
		add_settings_field( 'author_archive', __( 'Disable author archives', 'xo-security' ), array( $this, 'option_page_field_author_archive' ), 'xo_security_secret', 'xo_security_security_user_section' );
		add_settings_field( 'comment_author_class', __( 'Delete comment author class', 'xo-security' ), array( $this, 'option_page_field_comment_author_class' ), 'xo_security_secret', 'xo_security_security_user_section' );
		add_settings_field( 'oembed_author', __( 'Remove oEmbed username', 'xo-security' ), array( $this, 'option_page_field_oembed_author' ), 'xo_security_secret', 'xo_security_security_user_section' );
		add_settings_section( 'xo_security_disable_feed_section', __( 'RSS and Atom feeds', 'xo-security' ), '__return_false', 'xo_security_secret' );
		add_settings_field( 'disable_feed', __( 'Disable RSS and Atom feeds', 'xo-security' ), array( $this, 'option_page_field_disable_feed' ), 'xo_security_secret', 'xo_security_disable_feed_section' );
		add_settings_section( 'xo_security_security_generator_section', __( 'WordPress version', 'xo-security' ), '__return_false', 'xo_security_secret' );
		add_settings_field( 'remove_version', __( 'Delete version information', 'xo-security' ), array( $this, 'option_page_field_remove_version' ), 'xo_security_secret', 'xo_security_security_generator_section' );

		register_setting( 'xo_security_environment', 'xo_security_options', array( $this, 'environment_sanitize' ) );
		add_settings_section( 'xo_security_environment_section', __( 'Environment', 'xo-security' ), '__return_false', 'xo_security_environment' );
		add_settings_field( 'ip_mode', __( 'IP address mode', 'xo-security' ), array( $this, 'option_page_field_ip_mode' ), 'xo_security_environment', 'xo_security_environment_section' );
		add_settings_section( 'xo_security_loginlog_section', __( 'Login log', 'xo-security' ), '__return_false', 'xo_security_environment' );
		add_settings_field( 'auto_truncate', __( 'Automatic removal', 'xo-security' ), array( $this, 'option_page_field_auto_truncate' ), 'xo_security_environment', 'xo_security_loginlog_section' );
		add_settings_field( 'log_default_status', __( 'Status to display by default', 'xo-security' ), array( $this, 'option_page_field_log_default_status' ), 'xo_security_environment', 'xo_security_loginlog_section' );
	}

	function option_page_field_limit_count() {
		$interval = isset( $this->parent->options['interval'] ) ? $this->parent->options['interval'] : '0';
		echo '<select id="field_interval" name="xo_security_options[interval]">';
		echo '<option value="0"' . ($interval == '0' ? ' selected' : '') . '>' . __( 'No limit', 'xo-security' ) . '</option>';
		echo '<option value="1"' . ($interval == '1' ? ' selected' : '') . '>' . __( 'During the one-hour', 'xo-security' ) . '</option>';
		echo '<option value="12"' . ($interval == '12' ? ' selected' : '') . '>' . __( 'During the 12-hour', 'xo-security' ) . '</option>';
		echo '<option value="24"' . ($interval == '24' ? ' selected' : '') . '>' . __( 'During the 24-hour', 'xo-security' ) . '</option>';
		echo '<option value="48"' . ($interval == '48' ? ' selected' : '') . '>' . __( 'During the 48-hour', 'xo-security' ) . '</option>';
		echo '</select>';

		$limit_count = isset( $this->parent->options['limit_count'] ) ? $this->parent->options['limit_count'] : '4';
		echo '<p><input id="field_limit_count" name="xo_security_options[limit_count]" type="number" value="' . esc_attr( $limit_count ) . '" class="small-text" /> ' . __( 'times permission', 'xo-security' ) . '</p>';
	}

	function option_page_field_blocked_tarpit() {
		$blocked_tarpit = isset( $this->parent->options['blocked_tarpit'] ) ? $this->parent->options['blocked_tarpit'] : '30';
		echo '<p><input id="field_blocked_tarpit" name="xo_security_options[blocked_tarpit]" type="number" value="' . esc_attr( $blocked_tarpit ) . '" class="small-text" /> ' . __( 'sec (0-120)', 'xo-security' ) . '</p>';
	}

	function option_page_field_failed_tarpit() {
		$failed_tarpit = isset( $this->parent->options['failed_tarpit'] ) ? $this->parent->options['failed_tarpit'] : '3';
		echo '<p><input id="field_failed_tarpit" name="xo_security_options[failed_tarpit]" type="number" value="' . esc_attr( $failed_tarpit ) . '" class="small-text" /> ' . __( 'sec (0-10)', 'xo-security' ) . '</p>';
	}

	function option_page_field_login_alert() {
		$login_alert = isset( $this->parent->options['login_alert'] ) ? $this->parent->options['login_alert'] : false;
		$login_alert_admin_only = isset( $this->parent->options['login_alert_admin_only'] ) ? $this->parent->options['login_alert_admin_only'] : false;
		echo '<label for="field_login_alert" class="label-checkbox-toggle"><input id="field_login_alert" name="xo_security_options[login_alert]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $login_alert, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		$subject = isset( $this->parent->options['login_alert_subject'] ) ? $this->parent->options['login_alert_subject'] : __( 'Login at %SITENAME% site', 'xo-security' );
		echo '<p><label for="field_login_alert_subject">' . __( 'Subject:', 'xo-security' ) . '</label><br /><input id="field_login_alert_subject" name="xo_security_options[login_alert_subject]" type="text" value="' . esc_attr( $subject ) . '" class="regular-text" style="display: inline;" maxlength="100" /></p>';
		$body = isset( $this->parent->options['login_alert_body'] ) ? $this->parent->options['login_alert_body'] : __( '%USERNAME% logged in at %DATE% %TIME%.', 'xo-security' );
		echo '<p><label for="field_login_alert_body">' . __( 'Body:', 'xo-security' ) . '</label><br /><textarea id="field_login_alert_body" name="xo_security_options[login_alert_body]" class="large-text code" rows="3" col="50" />' . esc_attr( $body ) . '</textarea></p>';
		echo '<p class="description">' . __( 'In the Subject and Body, the following variables can be used: %SITENAME%, %USERNAME%, %DATE%, %TIME%, %IPADDRESS%, %USERAGENT%', 'xo-security' ) . '</p>';
		echo '<p><label for="field_login_alert_admin_only"><input id="field_login_alert_admin_only" name="xo_security_options[login_alert_admin_only]" type="checkbox" value="1" class="code" ' . checked( 1, $login_alert_admin_only, false ) . ' /> ' . __( 'Administrators only', 'xo-security' ) . '</label></p>';
	}

	function option_page_field_error_login_message() {
		$error_login_message = isset( $this->parent->options['error_login_message'] ) ? $this->parent->options['error_login_message'] : false;
		echo '<select id="field_password_reset" name="xo_security_options[error_login_message]">';
		echo '<option value="0"' . ( ! $error_login_message ? ' selected' : '' ) . '>' . __( 'Default', 'xo-security' ) . '</option>';
		echo '<option value="1"' . ( $error_login_message ? ' selected' : '' ) . '>' . __( 'Simplification', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_login_id_type() {
		$v = isset( $this->parent->options['login_id_type'] ) ? $this->parent->options['login_id_type'] : '';
		echo '<select id="field_login_id_type" name="xo_security_options[login_id_type]">';
		echo '<option value=""' . ( $v == '' ? ' selected' : '' ) . '>' . __( 'Username or Email Address (default)', 'xo-security' ) . '</option>';
		echo '<option value="username"' . ( $v == 'username' ? ' selected' : '' ) . '>' . __( 'Username only', 'xo-security' ) . '</option>';
		echo '<option value="email"' . ( $v == 'email' ? ' selected' : '' ) . '>' . __( 'Email Address only', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_login_captcha() {
		$login_captcha = isset( $this->parent->options['login_captcha'] ) ? $this->parent->options['login_captcha'] : '';
		echo '<select id="field_login_captcha" name="xo_security_options[login_captcha]">';
		echo '<option value=""' . ( $login_captcha == '' ? ' selected' : '' ) . '>' . __( 'Disabled', 'xo-security' ) . '</option>';
		echo '<option value="en"' . ( $login_captcha == 'en' ? ' selected' : '' ) . '>' . __( 'Alphanumeric', 'xo-security' ) . '</option>';
		echo '<option value="jp"' . ( $login_captcha == 'jp' ? ' selected' : '' ) . '>' . __( 'Hiragana (Japanese)', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_login_languages() {
		$login_languages = defined( 'XO_SECURITY_LANGUAGE_WHITE_LIST' ) ? XO_SECURITY_LANGUAGE_WHITE_LIST : '';

		if ( $login_languages ) {
			echo '<p><code>' . esc_html( $login_languages ) . '</code></p>';
		} else {
			$lang = strtolower( substr( $this->parent->get_language(), 0, 2 ) );
			echo '<p><code>' . __( 'Disabled', 'xo-security' ) . '</code></p>';
			echo '<p class="description">' . __( 'XO_SECURITY_LANGUAGE_WHITE_LIST set by a constant.', 'xo-security' ) . '</p>';
			echo '<p class="description">' . esc_html( sprintf( __( 'Example: define( \'XO_SECURITY_LANGUAGE_WHITE_LIST\', \'%s\' );', 'xo-security' ), $lang ) ) . '</p>';
		}
	}

	function option_page_field_auto_truncate() {
		$auto_truncate = isset( $this->parent->options['auto_truncate'] ) ? $this->parent->options['auto_truncate'] : '0';
		echo '<select id="field_auto_truncate" name="xo_security_options[auto_truncate]">';
		echo '<option value="0"' . ( $auto_truncate == 0 ? ' selected' : '' ) . '>' . __( 'Not automatic deletion', 'xo-security' ) . '</option>';
		echo '<option value="30"' . ( $auto_truncate == 30 ? ' selected' : '' ) . '>' . __( 'Older than 30 days', 'xo-security' ) . '</option>';
		echo '<option value="365"' . ( $auto_truncate >= 31 ? ' selected' : '' ) . '>' . __( 'Older than 365 days', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_log_default_status() {
		$log_default_status = isset( $this->parent->options['log_default_status'] ) ? $this->parent->options['log_default_status'] : '';
		echo '<select id="field_log_default_status" name="xo_security_options[log_default_status]">';
		echo '<option value=""' . ( $log_default_status == '' ? ' selected' : '' ) . '>' . __( 'All results', 'xo-security' ) . '</option>';
		echo '<option value="0"' . ( $log_default_status == '0' ? ' selected' : '' ) . '>' . __( 'Failure', 'xo-security' ) . '</option>';
		echo '<option value="1"' . ( $log_default_status >= '1' ? ' selected' : '' ) . '>' . __( 'Success', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_login_page() {
		$ms_common_login_page = isset( $this->parent->options['ms_common_login_page'] ) ? $this->parent->options['ms_common_login_page'] : false;

		if ( ! is_main_site() && $ms_common_login_page ) {
			echo '<p><code>' . __( 'Inherit main site settings', 'xo-security' ) . '</code></p>';
		} else {
			$c = isset( $this->parent->options['login_page'] ) ? $this->parent->options['login_page'] : false;
			echo '<label for="field_login_page" class="label-checkbox-toggle"><input id="field_login_page" name="xo_security_options[login_page]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $c, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
			$name = isset( $this->parent->options['login_page_name'] ) ? $this->parent->options['login_page_name'] : '';
			echo '<p><label for="field_login_page_name">' . __( 'Login file:', 'xo-security' ) . ' <input id="field_login_page_name" name="xo_security_options[login_page_name]" type="text" value="' . esc_attr( $name ) . '" class="regular-text" style="max-width:15em; display: inline;" maxlength="40" /></label>.php</p>';
			echo '<p>URL: <span id="login_url"></span></p>';
			echo '<p class="description">' . __( 'Characters can be used in the login file is only lowercase letters, numbers, hyphens and underscores.', 'xo-security' ) . '</p>';
			if ( is_multisite() && is_main_site() ) {
				echo '<p><label for="field_ms_common_login_page"><input id="field_ms_common_login_page" name="xo_security_options[ms_common_login_page]" type="checkbox" value="1" class="code" ' . checked( 1, $ms_common_login_page, false ) . ' /> ' . __( 'Common to all sites (recommended)', 'xo-security' ) . '</label></p>';
			}
		}
	}

	function option_page_field_password_reset() {
		$password_reset = isset( $this->parent->options['password_reset'] ) ? $this->parent->options['password_reset'] : true;
		echo '<select id="field_password_reset" name="xo_security_options[password_reset]">';
		echo '<option value="1"' . ( $password_reset ? ' selected' : '' ) . '>' . __( 'Enable', 'xo-security' ) . '</option>';
		echo '<option value="0"' . ( ! $password_reset ? ' selected' : '' ) . '>' . __( 'Disabled', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_login_site_link() {
		$login_site_link = isset( $this->parent->options['login_site_link'] ) ? $this->parent->options['login_site_link'] : true;
		echo '<select id="field_login_site_link" name="xo_security_options[login_site_link]">';
		echo '<option value="1"' . ( $login_site_link ? ' selected' : '' ) . '>' . __( 'Enable', 'xo-security' ) . '</option>';
		echo '<option value="0"' . ( ! $login_site_link ? ' selected' : '' ) . '>' . __( 'Disabled', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_comment_captcha() {
		$captcha = isset( $this->parent->options['comment_captcha'] ) ? $this->parent->options['comment_captcha'] : '';
		echo '<select id="field_comment_captcha" name="xo_security_options[comment_captcha]">';
		echo '<option value=""' . ( $captcha == '' ? ' selected' : '') . '>' . __( 'Disabled', 'xo-security' ) . '</option>';
		echo '<option value="en"' . ( $captcha == 'en' ? ' selected' : '') . '>' . __( 'Alphanumeric', 'xo-security' ) . '</option>';
		echo '<option value="jp"' . ( $captcha == 'jp' ? ' selected' : '') . '>' . __( 'Hiragana (Japanese)', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_comment_spam() {
		$spam = isset( $this->parent->options['comment_spam'] ) ? $this->parent->options['comment_spam'] : false;
		echo '<label for="field_comment_spam"><input id="field_comment_spam" name="xo_security_options[comment_spam]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $spam, false ) . ' /> ' . __( 'Does not include Japanese characters', 'xo-security' ) . '</label>';
		$message = isset( $this->parent->options['comment_spam_message'] ) ? $this->parent->options['comment_spam_message'] : __( 'Please enter comment in Japanese. (Anti-spam)', 'xo-security' );
		echo '<p><label for="field_comment_spam_message">'.__( 'Message to display under the comment field:', 'xo-security' ).'</label><br /><input id="field_comment_spam_message" name="xo_security_options[comment_spam_message]" type="text" value="' . esc_attr( $message ) . '" class="large-text" style="display: inline;" maxlength="100" /></p>';
	}

	function option_page_field_comment_spam_action() {
		$action = isset( $this->parent->options['comment_spam_action'] ) ? $this->parent->options['comment_spam_action'] : 'block';
		echo '<select id="field_comment_spam_action" name="xo_security_options[comment_spam_action]">';
		echo '<option value="block"' . ( $action == 'block' ? ' selected' : '') . '>' . __( 'Block', 'xo-security' ) . '</option>';
		echo '<option value="spam"' . ( $action == 'spam' ? ' selected' : '') . '>' . __( 'Save as spam', 'xo-security' ) . '</option>';
		echo '<option value="trash"' . ( $action == 'trash' ? ' selected' : '') . '>' . __( 'Save to trash', 'xo-security' ) . '</option>';
		echo '</select>';
	}

	function option_page_field_comment_bot() {
		$bot = isset( $this->parent->options['comment_bot'] ) ? $this->parent->options['comment_bot'] : false;
		echo '<label for="field_comment_bot" class="label-checkbox-toggle"><input id="field_comment_bot" name="xo_security_options[comment_bot]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $bot, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'If JavaScript is disabled, if a response is received within 1 second after the form is displayed, or if the form is unchecked, it will be considered a bot.', 'xo-security' ) . '</p>';
	}

	function option_page_field_xmlrpc() {
		$xmlrpc = isset( $this->parent->options['xmlrpc'] ) ? $this->parent->options['xmlrpc'] : false;
		echo '<label for="field_xmlrpc" class="label-checkbox-toggle"><input id="field_xmlrpc" name="xo_security_options[xmlrpc]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $xmlrpc, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
	}

	function option_page_field_pingback() {
		$pingback = isset( $this->parent->options['pingback'] ) ? $this->parent->options['pingback'] : false;
		echo '<label for="field_pingback" class="label-checkbox-toggle"><input id="field_pingback" name="xo_security_options[pingback]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $pingback, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
	}

	function option_page_field_rest() {
		$rest_server = rest_get_server();
		$namespaces = $rest_server->get_namespaces();
		$routes = array_keys( $rest_server->get_routes() );

		$rest = isset( $this->parent->options['rest'] ) ? $this->parent->options['rest'] : false;
		echo '<label for="field_rest" class="label-checkbox-toggle"><input id="field_rest" name="xo_security_options[rest]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $rest, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';

		$disable_endpoints = isset( $this->parent->options['rest_disable_endpoints'] ) ? explode( ',', $this->parent->options['rest_disable_endpoints'] ) : array();
		$counter = 1;

		echo '<p><div id="xo_security_rest_disable" class="xo-security-endpoints-checklist">';
		foreach ( $namespaces as $namespace ) {
			$namespace = '/' . $namespace;
			$counter++;

			echo '<ul>';
			echo '<li>';
			echo '<label class="selectit"><input id="field_rest_disable_' . esc_attr( $counter ) . '" class="namespace" name="xo_security_options[rest_disable_endpoints][]" value="' . esc_attr( $namespace ) . '" type="checkbox" ' . checked( in_array( $namespace, $disable_endpoints ), true, false ) . ' /> ' . esc_html( $namespace ) . '</label>';
			echo '<ul>';
			foreach ( $routes as $route ) {
				if ( $route != $namespace && strpos( $route, $namespace ) === 0 ) {
					$counter++;

					echo '<li>';
					echo '<label class="selectit"><input id="field_rest_disable_' . esc_attr( $counter ) . '" class="route" name="xo_security_options[rest_disable_endpoints][]" value="' . esc_attr( $route ) . '" type="checkbox" data-namespace="' . esc_attr( $namespace ) . '" ' . checked( in_array( $route, $disable_endpoints ), true, false ) . ' /> ' . esc_html( $route ) . '</label>';
					echo '</li>';
				}
			}
			echo '</ul>';
			echo '</li>';
			echo '</ul>';
		}
		echo "</div></p>\n";
		echo '<p class="description">' . __( 'Protect the checked endpoints from users who are not logged in.', 'xo-security' ) . '</p>';

		global $wp_version;
		if ( version_compare( $wp_version, '4.8', '<' ) ) {
			echo '<p class="description">' . __( 'Exclusion setting is valid only in WordPress version 4.7 or later.', 'xo-security' ) . '</p>';
		}
	}

	function option_page_field_rest_rename() {
		$c = isset( $this->parent->options['rest_rename'] ) ? $this->parent->options['rest_rename'] : false;
		echo '<label for="field_rest_rename" class="label-checkbox-toggle"><input id="field_rest_rename" name="xo_security_options[rest_rename]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $c, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		$name = !empty( $this->parent->options['rest_name'] ) ? $this->parent->options['rest_name'] : rest_get_url_prefix();
		echo '<p><label for="field_rest_name">' . __( 'Prefix:', 'xo-security' ) . ' <input id="field_rest_name" name="xo_security_options[rest_name]" type="text" value="' . esc_attr( $name ) . '" class="regular-text" style="max-width:15em; display: inline;" maxlength="40" /></label></p>';
		echo '<p class="description">' . __( 'Characters can be used in the prefix is only lowercase letters, numbers, hyphens and underscores.', 'xo-security' ) . '</p>';
	}

	function option_page_field_edit_author_slug() {
		$edit_author_slug = isset( $this->parent->options['edit_author_slug'] ) ? $this->parent->options['edit_author_slug'] : false;
		echo '<label for="field_edit_author_slug" class="label-checkbox-toggle"><input id="field_edit_author_slug" name="xo_security_options[edit_author_slug]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $edit_author_slug, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'The author slug (Nicename) item has been added to your profile screen for editing. The author slug is the nicename of the user data. The value is retained even if you remove the plugin.', 'xo-security' ) . '</p>';
	}

	function option_page_field_author_archive() {
		$author_archive = isset( $this->parent->options['author_archive'] ) ? $this->parent->options['author_archive'] : false;
		echo '<label for="field_author_archive" class="label-checkbox-toggle"><input id="field_author_archive" name="xo_security_options[author_archive]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $author_archive, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'The standard sitemap user provider is also removed.', 'xo-security' ) . '</p>';
	}

	function option_page_field_comment_author_class() {
		$comment_author_class = isset( $this->parent->options['comment_author_class'] ) ? $this->parent->options['comment_author_class'] : false;
		echo '<label for="field_comment_author_class" class="label-checkbox-toggle"><input id="field_comment_author_class" name="xo_security_options[comment_author_class]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $comment_author_class, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'Delete the class that contains the username that will be added to the comment list tag.', 'xo-security' ) . '</p>';
	}

	function option_page_field_oembed_author() {
		$oembed_author = isset( $this->parent->options['oembed_author'] ) ? $this->parent->options['oembed_author'] : false;
		echo '<label for="field_oembed_author" class="label-checkbox-toggle"><input id="field_oembed_author" name="xo_security_options[oembed_author]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $oembed_author, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'Remove the username (author_name and author_url) from the oEmbed response data.', 'xo-security' ) . '</p>';
	}

	function option_page_field_disable_feed() {
		$disable_feed = isset( $this->parent->options['disable_feed'] ) ? $this->parent->options['disable_feed'] : false;
		echo '<label for="field_disable_feed" class="label-checkbox-toggle"><input id="field_disable_feed" name="xo_security_options[disable_feed]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $disable_feed, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'Disable RSS and Atom feeds.', 'xo-security' ) . '</p>';
	}

	function option_page_field_remove_version() {
		$remove_version = isset( $this->parent->options['remove_version'] ) ? $this->parent->options['remove_version'] : false;
		echo '<label for="field_remove_version" class="label-checkbox-toggle"><input id="field_remove_version" name="xo_security_options[remove_version]" type="checkbox" value="1" class="code checkbox-toggle" ' . checked( 1, $remove_version, false ) . ' /> ' . __( 'ON', 'xo-security' ) . '</label>';
		echo '<p class="description">' . __( 'Remove the generator meta tags and WordPress versions such as links and script tags.', 'xo-security' ) . '</p>';
	}

	function option_page_field_ip_mode() {
		$ip_mode = isset( $this->parent->options['ip_mode'] ) ? $this->parent->options['ip_mode'] : '';

		$undefined_text = __( '(Undefined)', 'xo-security' );
		$xforwarded_ips = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? explode( ',', $_SERVER['HTTP_X_FORWARDED_FOR'] ) : array();
		$xforwarded_ips = array_pad( $xforwarded_ips, 3, $undefined_text );
		$remote_ip = isset( $_SERVER['REMOTE_ADDR'] ) ? $_SERVER['REMOTE_ADDR'] : $undefined_text;
		$xreal_ip = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? $_SERVER['HTTP_X_REAL_IP'] : $undefined_text;
		$client_ip = isset( $_SERVER['HTTP_CLIENT_IP'] ) ? $_SERVER['HTTP_CLIENT_IP'] : $undefined_text;

		echo '<fieldset><legend class="screen-reader-text"><span> ' . __( 'IP mode', 'xo-security' ) . ' </span></legend>';
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="" ' . checked( $ip_mode, '', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'Auto', 'xo-security' ) . '</span><code>' . esc_html( $this->parent->get_auto_ipaddress() ) . "</code></label><br />\n";
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="remote_addr" ' . checked( $ip_mode, 'remote_addr', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'REMOTE_ADDR', 'xo-security' ) . '</span><code>' . esc_html( $remote_ip ) . "</code></label><br />\n";
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="http_x_forward_for_1" ' . checked( $ip_mode, 'http_x_forward_for_1', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'X_FORWARDED_FOR: Client', 'xo-security' ) . '</span><code>' . esc_html( $xforwarded_ips[0] ) . "</code></label><br />\n";
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="http_x_forward_for_2" ' . checked( $ip_mode, 'http_x_forward_for_2', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'X_FORWARDED_FOR: Proxy1', 'xo-security' ) . '</span><code>' . esc_html( $xforwarded_ips[1] ) . "</code></label><br />\n";
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="http_x_forward_for_3" ' . checked( $ip_mode, 'http_x_forward_for_3', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'X_FORWARDED_FOR: Proxy2', 'xo-security' ) . '</span><code>' . esc_html( $xforwarded_ips[2] ) . "</code></label><br />\n";
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="http_x_real_ip" ' . checked( $ip_mode, 'http_x_real_ip', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'HTTP_X_REAL_IP', 'xo-security' ) . '</span><code>' . esc_html( $xreal_ip ) . "</code></label><br />\n";
		echo '<label><input name="xo_security_options[ip_mode]" type="radio" value="http_client_ip" ' . checked( $ip_mode, 'http_client_ip', false ) . ' /> ';
		echo '<span class="ip-mode-text">' . __( 'HTTP_CLIENT_IP', 'xo-security' ) . '</span><code>' . esc_html( $client_ip ) . "</code></label><br />\n";
		echo '</fieldset>';
		echo '<p class="description">' . __( 'Normally you should select "Auto".', 'xo-security' ) . '</p>';
	}

	function login_sanitize( $input ) {
		global $option_page, $wp_filesystem;
		
		if ( $option_page !== 'xo_security_login' ) {
			return $input;
		}
		
		$output = $this->get_option();

		$input['interval'] = isset( $input['interval'] ) ? intval( $input['interval'] ) : 0;
		$input['limit_count'] = isset( $input['limit_count'] ) ? intval( $input['limit_count'] ) : 4;
		if ( $input['interval'] > 0 ) {
			if ( 0 >= $input['limit_count'] || $input['limit_count'] > 100 ) {
				add_settings_error( 'xo_security', 'limit_count', __( 'Attempts to limit the number of times enter numbers from 1 to 100.', 'xo-security' ) );
			}
		}

		$input['blocked_tarpit'] = isset( $input['blocked_tarpit'] ) ? intval( $input['blocked_tarpit'] ) : 30;
		if ( 0 > $input['blocked_tarpit'] || $input['blocked_tarpit'] > 120 ) {
			add_settings_error( 'xo_security', 'blocked_tarpit', __( 'In the numbers from 0 to 120 block when the response delay.', 'xo-security' ) );
		}

		$input['failed_tarpit'] = isset( $input['failed_tarpit'] ) ? intval( $input['failed_tarpit'] ) : 3;
		if ( 0 > $input['failed_tarpit'] || $input['failed_tarpit'] > 10 ) {
			add_settings_error( 'xo_security', 'failed_tarpit', __( 'Failure response delay enter in the numbers from 0 to 10.', 'xo-security' ) );
		}

		$input['login_page'] = isset( $input['login_page'] ) ? $input['login_page'] : false;
		$input['login_page_name'] = isset( $input['login_page_name'] ) ? $input['login_page_name'] : '';
		if ( ( $input['login_page'] !== $output['login_page'] ) || ( $input['login_page_name'] !== $output['login_page_name'] ) ) {
			$url = wp_nonce_url( 'options-general.php?page=xo-security-settings', 'xo-security-settings' );
			if ( false === ( $creds = request_filesystem_credentials( $url, '', false, false, null ) ) ) {
				add_settings_error( 'xo_security', 'file', __( 'Unable to connect to the filesystem.', 'xo-security' ) );
			} else {
				if ( ! WP_Filesystem( $creds ) ) {
					add_settings_error( 'xo_security', 'file', __( 'Unable to write.', 'xo-security' ) );
				} else {
					if ( $output['login_page_name'] ) {
						$old_path = ABSPATH . $output['login_page_name'] . '.php';
						if ( $wp_filesystem->exists( $old_path ) ) {
							$wp_filesystem->delete( $old_path );
						}
					}
					if ( $input['login_page'] ) {
						if ( $input['login_page_name'] ) {
							$new_path = ABSPATH . $input['login_page_name'] . '.php';
							if ( $wp_filesystem->exists( $new_path ) ) {
								$input['login_page'] = $output['login_page'];
								$input['login_page_name'] = $output['login_page_name'];
								add_settings_error( 'xo_security', 'login_page_name', __( 'The file specified in the login file already exists. Please enter a different name.', 'xo-security' ) );
							} else {
								$result = $wp_filesystem->put_contents( $new_path, stripslashes( $this->loginfile_content ), FS_CHMOD_FILE );
								if ( ! $result || ! $wp_filesystem->exists( $new_path ) ) {
									$input['login_page'] = $output['login_page'];
									$input['login_page_name'] = $output['login_page_name'];
									add_settings_error( 'xo_security', 'file', __( 'Failed to create a login file.', 'xo-security' ) );
								}
							}
						} else {
							$input['login_page'] = $output['login_page'];
							$input['login_page_name'] = $output['login_page_name'];
							add_settings_error( 'xo_security', 'login_page_name', __( 'It is not possible to omit the login file.', 'xo-security' ) );
						}
					} else {
						$input['login_page_name'] = '';
					}
				}
			}
		}

		$output['interval'] = $input['interval'];
		$output['limit_count'] = $input['limit_count'];
		$output['blocked_tarpit'] = $input['blocked_tarpit'];
		$output['failed_tarpit'] = $input['failed_tarpit'];
		$output['login_page'] = $input['login_page'];
		$output['login_page_name'] = $input['login_page_name'];
		$output['ms_common_login_page'] = isset( $input['ms_common_login_page'] ) ? $input['ms_common_login_page'] : false;
		$output['login_alert'] = isset( $input['login_alert'] ) ? $input['login_alert'] : false;
		$output['login_alert_subject'] = isset( $input['login_alert_subject'] ) ? $input['login_alert_subject'] : null;
		$output['login_alert_body'] = isset( $input['login_alert_body'] ) ? $input['login_alert_body'] : null;
		$output['login_alert_admin_only'] = isset( $input['login_alert_admin_only'] ) ? $input['login_alert_admin_only'] : false;
		$output['login_captcha'] = $input['login_captcha'];
		$output['error_login_message'] = ( $input['error_login_message'] == '1' );
		$output['login_id_type'] = $input['login_id_type'];
		$output['password_reset'] = ( $input['password_reset'] == '1' );
		$output['login_site_link'] = ( $input['login_site_link'] == '1' );

		return $output;
	}

	function comment_sanitize( $input ) {
		global $option_page;

		if ( $option_page !== 'xo_security_comment' ) {
			return $input;
		}

		$output = get_option( 'xo_security_options' );

		$output['comment_captcha'] = $input['comment_captcha'];
		$output['comment_spam'] = isset( $input['comment_spam'] );
		$output['comment_spam_message'] = $input['comment_spam_message'];
		$output['comment_spam_action'] = $input['comment_spam_action'];
		$output['comment_bot'] = isset( $input['comment_bot'] );

		return $output;
	}

	function xmlrpc_sanitize( $input ) {
		global $option_page;

		if ( $option_page !== 'xo_security_xmlrpc' ) {
			return $input;
		}

		$output = get_option( 'xo_security_options' );

		$output['xmlrpc'] = isset( $input['xmlrpc'] ) ? $input['xmlrpc'] : false;
		$output['pingback'] = isset( $input['pingback'] ) ? $input['pingback'] : false;

		return $output;
	}

	function restapi_sanitize( $input ) {
		global $option_page;

		if ( $option_page !== 'xo_security_restapi' ) {
			return $input;
		}

		$output = get_option( 'xo_security_options' );

		$input['rest_rename'] = isset( $input['rest_rename'] ) ? $input['rest_rename'] : false;
		$input['rest_name'] = isset( $input['rest_name'] ) ? $input['rest_name'] : '';
		if (
			( !isset( $output['rest_rename'] ) || $input['rest_rename'] !== $output['rest_rename'] ) ||
			( !isset( $output['rest_name'] ) || $input['rest_name'] !== $output['rest_name'] )
		) {
			set_transient( 'xo_security_flush_rewrite_rules', true, MINUTE_IN_SECONDS );
		}

		$output['rest'] = isset( $input['rest'] ) ? $input['rest'] : false;
		if ( isset( $input['rest_disable_endpoints'] ) && is_array( $input['rest_disable_endpoints'] ) ) {
			$output['rest_disable_endpoints'] = implode( ',', $input['rest_disable_endpoints'] );
		} else {
			$output['rest_disable_endpoints'] = '';
		}
		$output['rest_rename'] = $input['rest_rename'];
		$output['rest_name'] = $input['rest_name'];

		return $output;	
	}

	function secret_sanitize( $input ) {
		global $option_page;

		if ( $option_page !== 'xo_security_secret' ) {
			return $input;
		}

		$output = get_option( 'xo_security_options' );

		$author_archive = isset( $input['author_archive'] );
		if ( ! isset( $output['author_archive'] ) || $author_archive !== $output['author_archive'] ) {
			if ( $author_archive ) {
				add_filter( 'author_rewrite_rules', array( $this, 'author_rewrite_rules' ) );
			} else {
				remove_filter( 'author_rewrite_rules', array( $this, 'author_rewrite_rules' ) );
			}
			set_transient( 'xo_security_flush_rewrite_rules', true, MINUTE_IN_SECONDS );
		}

		$output['edit_author_slug'] = isset( $input['edit_author_slug'] );
		$output['author_archive'] = $author_archive;
		$output['comment_author_class'] = isset( $input['comment_author_class'] );
		$output['oembed_author'] = isset( $input['oembed_author'] );
		$output['disable_feed'] = isset( $input['disable_feed'] );
		$output['remove_version'] = isset( $input['remove_version'] );

		return $output;
	}

	function environment_sanitize( $input ) {
		global $wpdb, $option_page;

		if ( $option_page !== 'xo_security_environment' ) {
			return $input;
		}

		$output = get_option( 'xo_security_options' );

		if ( $input['auto_truncate'] > 0 ) {
			$wpdb->query( $wpdb->prepare( "DELETE FROM `{$this->parent->loginlog_table}` WHERE `login_time` <= DATE_SUB(NOW(), INTERVAL %d day)", $input['auto_truncate'] ) );
		}

		$output['ip_mode'] = $input['ip_mode'];
		$output['auto_truncate'] = $input['auto_truncate'];
		$output['log_default_status'] = $input['log_default_status'];

		return $output;
	}

	function dashboard_setup() {
		wp_add_dashboard_widget( 'xo_security_dashboard_login_widget', __( 'Login information', 'xo-security' ), array( $this, 'dashboard_login_widget' ) );
	}

	function dashboard_login_widget() {
		global $wpdb;
		global $current_user;

		$datetime_format = get_option( 'date_format' ) . ' ' . get_option( 'time_format' );
		$user_login = $current_user->get( 'user_login' );

		$current_date = $wpdb->get_var(
			"SELECT `login_time` FROM `{$this->parent->loginlog_table}` WHERE `user_name`='{$user_login}' AND `success`=1 ORDER BY `login_time` DESC LIMIT 1;"
		);
		$last_date = $wpdb->get_var(
			"SELECT `login_time` FROM `{$this->parent->loginlog_table}` WHERE `user_name`='{$user_login}' AND `success`=1 ORDER BY `login_time` DESC LIMIT 1,2;"
		);

		echo '<div class="login_widget">';
		echo '<ul>';
		echo '<li>' . __( 'Current login date', 'xo-security' ) . ': ' .  ( $current_date === null ? __( 'Unknown', 'xo-security' ) : esc_html( mysql2date( $datetime_format, $current_date ) ) ) . '</li>';
		echo '<li>' . __( 'Last login date', 'xo-security' ) . ': ' .  ( $last_date === null ? __( 'Unknown', 'xo-security' ) : esc_html( mysql2date( $datetime_format, $last_date ) ) ). '</li>';
		echo '</ul>';
		echo '</div>' . "\n";

		if ( $current_user->has_cap( 'administrator' ) ) {
			$timestamp = current_time( 'timestamp' );
			$interval_hour = isset( $this->parent->options['interval'] ) ? (int)$this->parent->options['interval'] : 0;

			$hour = $interval_hour > 0 ? $interval_hour : 24;
			$time = date( 'Y-m-d H:i:s', (int)$timestamp - ( $hour * 60 * 60 ) );
			$failed_recent_count = $wpdb->get_var(
				"SELECT COUNT(`ip_address`) FROM `{$this->parent->loginlog_table}` WHERE `success`=0 AND `login_time`>'{$time}';"
			);

			$first_date = $wpdb->get_var(
				"SELECT `login_time` FROM `{$this->parent->loginlog_table}` ORDER BY `login_time` ASC LIMIT 1;"
			);
			if ( $first_date === null ) {
				$day = 0;
				$failed_count = 0;
			} else {
				$day = (int)(abs( $timestamp - strtotime( $first_date ) ) / ( 60 * 60 * 24 ) );
				$failed_count = $wpdb->get_var(
					"SELECT COUNT(`ip_address`) FROM `{$this->parent->loginlog_table}` WHERE `success`=0;"
				);
			}

			echo '<div class="login_widget">';
			echo '<ul>';
			echo '<li>' . esc_html( sprintf( _n( 'Failed login count (%d hour)', 'Failed login count (%d hours)', $hour, 'xo-security' ), $hour ) ) . ': ' . esc_html( $failed_recent_count ) . '</li>';
			echo '<li>' . esc_html( sprintf( _n( 'Failed login count (%d day)', 'Failed login count (%d days)', $day, 'xo-security' ), $day ) ) . ': ' .  esc_html( $failed_count ) . '</li>';
			echo '</ul>';
			echo '</div>' . "\n";
		}
	}
}
